﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization.Operators.LCS;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.DecisionList {
  [Item("DecisionListCreator", "Description missing")]
  [StorableClass]
  public abstract class DecisionListCreator : SingleSuccessorOperator, IDecisionListCreator {

    #region Parameter Properties
    public IValueLookupParameter<IDecisionListClassificationProblemData> ProblemDataParameter {
      get { return (IValueLookupParameter<IDecisionListClassificationProblemData>)Parameters["ProblemData"]; }
    }
    public IValueLookupParameter<IntValue> InitialNumberOfRulesParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["InitialNumberOfRules"]; }
    }
    public IValueLookupParameter<PercentValue> OneProbabilityParameter {
      get { return (IValueLookupParameter<PercentValue>)Parameters["OneProbability"]; }
    }
    public IValueLookupParameter<ItemCollection<IDiscretizer>> DiscretizersParameter {
      get { return (IValueLookupParameter<ItemCollection<IDiscretizer>>)Parameters["Discretizers"]; }
    }
    public ILookupParameter<DecisionList> DecisionListParameter {
      get { return (ILookupParameter<DecisionList>)Parameters["DecisionList"]; }
    }
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<IGAssistNiche> GAssistNicheParameter {
      get { return (ILookupParameter<IGAssistNiche>)Parameters["GAssistNiche"]; }
    }
    #endregion

    [StorableConstructor]
    protected DecisionListCreator(bool deserializing) : base(deserializing) { }
    protected DecisionListCreator(DecisionListCreator original, Cloner cloner)
      : base(original, cloner) {
    }
    public DecisionListCreator()
      : base() {
      Parameters.Add(new ValueLookupParameter<IDecisionListClassificationProblemData>("ProblemData", ""));
      Parameters.Add(new ValueLookupParameter<IntValue>("InitialNumberOfRules", ""));
      Parameters.Add(new ValueLookupParameter<PercentValue>("OneProbability", ""));
      Parameters.Add(new ValueLookupParameter<ItemCollection<IDiscretizer>>("Discretizers", ""));
      Parameters.Add(new LookupParameter<DecisionList>("DecisionList", ""));
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator which should be used for stochastic manipulation operators."));
      Parameters.Add(new LookupParameter<IGAssistNiche>("GAssistNiche", ""));
    }

    public override IOperation Apply() {
      if (GAssistNicheParameter.ActualValue != null && !(GAssistNicheParameter.ActualValue is IAction)) throw new ArgumentException("GAssistNiche has to be a IAction.");
      DecisionListParameter.ActualValue = Create(RandomParameter.ActualValue,
                                              ProblemDataParameter.ActualValue.SampleRuleParameter.Value,
                                              InitialNumberOfRulesParameter.ActualValue.Value,
                                              OneProbabilityParameter.ActualValue.Value,
                                              DiscretizersParameter.ActualValue,
                                              (IAction)GAssistNicheParameter.ActualValue);
      return base.Apply();
    }

    protected abstract DecisionList Create(IRandom random, Rule sampleRule, int initialNumberOfRules, double oneProbability, ItemCollection<IDiscretizer> discretizers, IAction niche);
  }
}
