﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization.Operators.LCS;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.DecisionList {
  [StorableClass]
  [Item("DecisionListInput", "")]
  public class DecisionListInput : Item, IGAssistInput {

    [Storable]
    private Dictionary<string, string> inputDictionary;
    public Dictionary<string, string> InputDictionary {
      get { return inputDictionary; }
      protected set {
        inputDictionary = value;
      }
    }

    public IOrderedEnumerable<string> Order { get { return InputDictionary.Keys.OrderBy(x => x); } }

    [StorableConstructor]
    protected DecisionListInput(bool deserializing) { }
    protected DecisionListInput(DecisionListInput original, Cloner cloner) {
      InputDictionary = new Dictionary<string, string>(original.InputDictionary.Count);
      foreach (var item in original.InputDictionary) {
        InputDictionary.Add(item.Key, item.Value);
      }
    }
    public DecisionListInput()
      : base() {
      InputDictionary = new Dictionary<string, string>();
    }
    public DecisionListInput(int capacity)
      : base() {
      InputDictionary = new Dictionary<string, string>(capacity);
    }
    public DecisionListInput(IDictionary<string, string> dictionary)
      : base() {
      InputDictionary = new Dictionary<string, string>(dictionary);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new DecisionListInput(this, cloner);
    }

    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      bool first = true;
      foreach (var item in InputDictionary) {
        if (first) {
          first = false;
        } else {
          sb.Append("; ");
        }
        sb.Append(String.Format("{0}: {1}", item.Key, item.Value));
      }
      return sb.ToString();
    }

    #region IGAssistInput Members
    public IEnumerable<string> VariableNames {
      get { return inputDictionary.Keys; }
    }
    public string GetVariableValue(string variableName) {
      return inputDictionary[variableName];
    }
    #endregion
  }
}
